import os
import shutil


class Report(object):
    def __init__(self, results_dir, parent):
        self.results_dir = results_dir
        self.fn = os.path.join(results_dir, 'results.html')
        self.templates_dir = os.path.join(results_dir, '../../', 'templates')

        self.set_statics()

    def set_statics(self):
        if not os.path.exists(self.results_dir):
            return
        try:
            shutil.copytree(os.path.join(self.templates_dir, 'css'), os.path.join(self.results_dir, 'css'))
            shutil.copytree(os.path.join(self.templates_dir, 'img'), os.path.join(self.results_dir, 'img'))
            shutil.copytree(os.path.join(self.templates_dir, 'scripts'), os.path.join(self.results_dir, 'scripts'))
        except OSError:
            raise OSError("Could not create directory for results")

    def write_report(self, template):
        with open(self.fn, 'w') as f:
            f.write(template)



import os
import time
from jinja2 import Environment, FileSystemLoader
from oct.results import graphs
from oct.results.reportresults import Results, ReportResults
from oct.results.reportwriter import Report


def output(results_dir, results_file, config, parent='../../'):
    """Write the results output for the given test

    :param results_dir str: the directory for the results
    :param results_file str: the file for the results
    :param run_time int: the total test time elapsed
    :param rampup int: the rampup setting for the test
    :param ts_interval int: the interval in second setting for the test
    :param turrets: the turrets configuration
    :param parents str: the parent directory
    """
    start = time.time()
    results_dir = os.path.abspath(results_dir)
    results = Results(os.path.join(results_dir, results_file), config['run_time'])

    if len(results.resp_stats_list) == 0:
        print("No results, cannot create report")
        return False

    print('transactions: %i' % results.total_transactions)
    print('errors: %i' % results.total_errors)
    print('')
    print('test start: %s' % results.start_datetime)
    print('test finish: %s' % results.finish_datetime)
    print('')

    report_results = ReportResults(results, config['results_ts_interval'])
    report_results.set_all_transactions_results()

    data = {
        'report': report_results,
        'run_time': config['run_time'],
        'ts_interval': config['results_ts_interval'],
        'turrets_config': results.turrets,
        'all_results': report_results.all_results,
    }

    graphs.resp_graph_raw(report_results.all_results['trans_timer_points'], 'All_Transactions_response_times.svg',
                          results_dir)
    graphs.resp_graph(report_results.all_results['interval_results'].avg_resptime_points,
                      report_results.all_results['interval_results'].percentile_80,
                      report_results.all_results['interval_results'].percentile_90,
                      'All_Transactions_response_times_intervals.svg',
                      results_dir)
    graphs.tp_graph(report_results.all_results['throughput_points'],
                    'All_Transactions_throughput.svg',
                    results_dir)

    report_results.clear_all_transactions()
    report_results.set_custom_timers()

    for timer in report_results.custom_timers:
        graphs.resp_graph_raw(timer['trans_timer_points'], timer['name'] + '_response_times.svg', results_dir)
        graphs.tp_graph(timer['throughput_points'], timer['name'] + '_throughput.svg', results_dir)
        graphs.resp_graph(timer['interval_results'].avg_resptime_points,
                          timer['interval_results'].percentile_80,
                          timer['interval_results'].percentile_90,
                          timer['name'] + '_response_times_intervals.svg',
                          results_dir)

    # generate the jinja template
    j_env = Environment(loader=FileSystemLoader(os.path.join(results_dir, parent, 'templates')))
    template = j_env.get_template('report.html')

    report = Report(results_dir, parent)
    report.write_report(template.render(data))
    print("Report generated in {} seconds".format(time.time() - start))
    return True



import os
import sys
import json

from oct.results.models import Result, Turret, set_database, db


class ResultsWriter(object):
    """This class will handle results and stats comming from the turrets

    :param output_dir: the output directory for the results
    :type output_dir: str
    """
    def __init__(self, output_dir, config):
        self.output_dir = output_dir
        self.trans_count = 0
        self.timer_count = 0
        self.error_count = 0
        self.turret_name = 'Turret'
        self.results = []

        try:
            os.makedirs(self.output_dir, 0o755)
        except OSError:
            sys.stderr.write("ERROR: Can not create output directory\n")
            sys.exit(1)

        set_database(self.output_dir + "results.sqlite", db, config)
        db.connect()
        db.create_tables([Result, Turret])

    def write_turret(self, datas):
        turret = Turret(name=datas['turret'], canons=datas['canons'], script=datas['script'], rampup=datas['rampup'],
                        uuid=datas['uuid'], status=datas['status'])
        turret.save()
        return turret

    def write_result(self, datas):
        self.trans_count += 1
        self.timer_count += len(datas['custom_timers'])
        if datas['error']:
            self.error_count += 1

        datas['custom_timers'] = json.dumps(datas['custom_timers'])
        self.results.append(datas)

        if len(self.results) >= 450:  # SQLite limit for inser_many is 500
            with db.atomic():
                Result.insert_many(self.results).execute()
            del self.results[:]

    def write_remaining(self):
        with db.atomic():
            Result.insert_many(self.results).execute()
        del self.results[:]



from oct_turrets.base import BaseTransaction
import random
import time


class Transaction(BaseTransaction):
    def __init__(self):
        pass

    def setup(self):
        """Setup data or objects here
        """
        pass

    def run(self):
        r = random.uniform(1, 2)
        time.sleep(r)
        self.custom_timers['Example_Timer'] = r

    def tear_down(self):
        """Clear cache or reset objects, etc. Anything that must be done after
        the run method and before its next execution
        """
        pass


if __name__ == '__main__':
    trans = Transaction()
    trans.run()
    print(trans.custom_timers)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#  Copyright (c) 2010-2012 Corey Goldberg (corey@goldb.org)
#  License: GNU LGPLv3
#
#  This file is part of Multi-Mechanize | Performance Test Framework
#
from __future__ import print_function
import argparse
import os
import shutil
import time
from datetime import datetime
from oct.results.resultsoutput import output as output_results

import oct.results.resultswriter as resultswriter
from oct.utilities.configuration import configure
from oct.core.hq import HightQuarter


def main():
    """
    Main function to run oct tests.
    """

    parser = argparse.ArgumentParser(prog="oct-run")
    parser.add_argument('project_name', help="The project directory")
    parser.add_argument('-r', '--results', dest='results_dir', help='results directory to reprocess')
    parser.add_argument('-d', '--directory', dest='project_dir', help='directory containing project folder',
                        default='.')
    args = parser.parse_args()

    run(args)


def run(cmd_args):

    project_name = cmd_args.project_name
    config = configure(project_name, cmd_args)

    run_localtime = time.localtime()
    milisec = datetime.now().microsecond
    output_dir = '%s/%s/results/results_%s' % (cmd_args.project_dir, project_name,
                                               time.strftime('%Y.%m.%d_%H.%M.%S_' + str(milisec) + '/', run_localtime))

    # this queue is shared between all processes/threads
    rw = resultswriter.ResultsWriter(output_dir, config)

    script_prefix = os.path.join(cmd_args.project_dir, project_name, "test_scripts")
    script_prefix = os.path.normpath(script_prefix)

    hq = HightQuarter(config.get('publish_port', 5000), config.get('rc_port', 5001), rw, config)
    hq.wait_turrets(config.get("min_turrets", 1))
    hq.run()

    # all agents are done running at this point
    time.sleep(.2)  # make sure the writer queue is flushed
    print('\n\nanalyzing results...\n')
    if output_results(output_dir, 'results.sqlite', config):
        print('created: %sresults.html\n' % output_dir)

    # copy config file to results directory
    project_config = os.sep.join([cmd_args.project_dir, project_name, 'config.json'])
    saved_config = os.sep.join([output_dir, 'config.json'])
    shutil.copy(project_config, saved_config)

    print('done.\n')

    return



import argparse
import tarfile
import os.path
import tempfile
from jinja2 import Environment, PackageLoader

from oct.utilities.configuration import configure_for_turret


def main():
    parser = argparse.ArgumentParser(description='Give parameters for package a turret config_file and v_user')
    parser.add_argument('path', type=str, help='path for oct project dir')
    args = parser.parse_args()

    if os.path.exists(args.path):
        env = Environment(loader=PackageLoader('oct.utilities', 'templates'))
        turret_config = env.get_template('configuration/turret_config.json')
        turret_setup = env.get_template('scripts/setup.py')

        config_file = os.path.join(os.path.abspath(args.path), "config.json")
        configs = configure_for_turret(args.path, config_file)

        for turret in configs:
            content = turret_config.render(turret)
            tmp_file = os.path.join(tempfile.gettempdir(), turret['name'] + ".json")
            tmp_setup = os.path.join(tempfile.gettempdir(), turret['name'] + ".py")
            turrets_requirements = turret.get('turrets_requirements', [])
            setup_file = turret_setup.render({'turrets_requirements': turrets_requirements, 'name': turret.get('name')})
            try:
                with open(tmp_file, 'w') as f:
                    f.write(content)
                with open(tmp_setup, 'w') as f:
                    f.write(setup_file)
                pack_turret(turret, tmp_file, tmp_setup, os.path.dirname(config_file), args.path)
                os.remove(tmp_file)
                os.remove(tmp_setup)
            except IOError as e:
                print("Error while packaging turret %s" % turret['name'])
                print("Error: %s" % e)
    else:
        parser.error("you need to enter a valid project path")


def pack_turret(turret_config, tmp_config_file, tmp_setup, base_config_path, path=None):
    """pack a turret into a tar file based on the turret configuration

    :param turret_config dict: the turret configuration to pack
    :param tmp_config_file str: the path of the temp config file
    :param base_config_path str: the base directory of the main configuration file
    """
    file_name = turret_config['name']
    if path is not None:
        file_name = os.path.join(path, file_name)
    tar_file = tarfile.open(file_name + ".tar", 'w')
    tar_file.add(os.path.abspath(tmp_config_file), arcname="config.json")
    tar_file.add(os.path.abspath(tmp_setup), arcname="setup.py")

    script_path = os.path.join(os.path.abspath(base_config_path), turret_config['script'])
    tar_file.add(script_path, arcname=turret_config['script'])
    for f in tar_file.getnames():
        print("Added %s" % f)
    tar_file.close()
    print("Archive %s created" % (file_name + ".tar"))
    print("=========================================")



from __future__ import print_function
import zmq
import time
import json
import traceback


class HightQuarter(object):
    """The main hight quarter that will receive informations from the turrets
    and send the start message

    :param publish_port int: the port for publishing information to turrets
    :param rc_port int: the result collector port for collecting results from the turrets
    :param results_writer ResultsWriter: the results writer
    :param config dict: the configuration of the test
    """
    def __init__(self, publish_port, rc_port, results_writer, config):
        self.context = zmq.Context()
        self.poller = zmq.Poller()

        self.result_collector = self.context.socket(zmq.PULL)
        self.result_collector.bind("tcp://*:{}".format(rc_port))

        self.publisher = self.context.socket(zmq.PUB)
        self.publisher.bind("tcp://*:{}".format(publish_port))

        self.poller.register(self.result_collector, zmq.POLLIN)

        self.results_writer = results_writer
        self.config = config
        self.turrets = []
        self.started = False

        # waiting for init sockets
        print("Warmup")
        time.sleep(1)

    def _turret_already_exists(self, turret_data):
        for t in self.turrets:
            if turret_data['uuid'] == t.uuid:
                return True
        return False

    def _add_turret(self, turret_data):
        self.turrets.append(self.results_writer.write_turret(turret_data))
        if self.started:
            self._publish({'command': 'start', 'msg': 'open fire'})

    def _update_turret(self, turret_data):
        for t in self.turrets:
            if turret_data['uuid'] == t.uuid:
                t.status = turret_data['status']
                t.save()
                break

    def _publish(self, message, channel=''):
        data = json.dumps(message)
        self.publisher.send_string("%s %s" % (channel, data))

    def _process_turret_status(self, data):
        if 'status' in data:
            if self._turret_already_exists(data):
                self._update_turret(data)
            else:
                self._add_turret(data)
            return True
        else:
            return False

    def wait_turrets(self, wait_for):
        """Wait until wait_for turrets are connected and ready
        """
        print("waiting for {} turrets to connect".format(wait_for - len(self.turrets)))
        while len(self.turrets) < wait_for:
            self._publish({'command': 'status_request', 'msg': None})
            socks = dict(self.poller.poll(2000))
            if self.result_collector in socks:
                data = self.result_collector.recv_json()
                self._process_turret_status(data)
                print("waiting for {} turrets to connect".format(wait_for - len(self.turrets)))

    def run(self):
        """Run the hight quarter, lunch the turrets and wait for results
        """
        elapsed = 0
        start_time = time.time()
        self._publish({'command': 'start', 'msg': 'open fire'})
        self.started = True
        display = 'turrets: {}, elapsed: {}   transactions: {}  timers: {}  errors: {}\r'
        while elapsed <= (self.config['run_time']):
            try:
                socks = dict(self.poller.poll(1000))
                if self.result_collector in socks:
                    data = self.result_collector.recv_json()
                    if 'status' not in data:
                        self.results_writer.write_result(data)
                    else:
                        self._process_turret_status(data)
                print(display.format(len(self.turrets), round(elapsed), self.results_writer.trans_count,
                                     self.results_writer.timer_count,
                                     self.results_writer.error_count), end='')
                elapsed = time.time() - start_time
            except (Exception, KeyboardInterrupt):
                print("\nStopping test, sending stop command to turrets")
                self._publish({'command': 'stop', 'msg': 'premature stop'})
                traceback.print_exc()
                break
        self._publish({'command': 'stop', 'msg': 'stopping fire'})
        print("\n\nProcessing all remaining messages...")
        self.result_collector.unbind(self.result_collector.LAST_ENDPOINT)
        try:
            data = self.result_collector.recv_json(zmq.NOBLOCK)
            while data:
                data = self.result_collector.recv_json(zmq.NOBLOCK)
                if 'status' not in data:
                    self.results_writer.write_result(data)
        except zmq.Again:
            self.result_collector.close()
            self.publisher.close()
            self.results_writer.write_remaining()



import os
import csv
import argparse

from oct.results.models import db, Result, set_database


def results_to_csv(result_file, output_file, delimiter=';'):
    """Take a sqlite filled database of results and return a csv file

    :param result_file str: the path of the sqlite database
    :param output_file str: the path of the csv output file
    :param delimiter str: the desired delimiter for the output csv file
    """
    if not os.path.isfile(result_file):
        raise OSError("Results file does not exists")
    headers = ['elapsed', 'epoch', 'turret_name', 'scriptrun_time', 'error']
    headers_row = {}

    set_database(result_file, db, {})

    results = Result.select()

    for item in results:
        result_item = item.to_dict()
        for k in result_item['custom_timers'].keys():
            if k not in headers:
                headers.append(k)
                headers_row[k] = k

    with open(output_file, "w+") as f:
        writer = csv.DictWriter(f, fieldnames=headers, delimiter=delimiter)
        headers_row.update({
            'elapsed': 'elapsed time',
            'epoch': 'epoch (in seconds)',
            'turret_name': 'turret name',
            'scriptrun_time': 'transaction time',
            'error': 'error'
        })
        writer.writerow(headers_row)
        for result_item in results:
            line = result_item.to_dict()
            for key, value in line['custom_timers'].items():
                line[key] = value
            del line['custom_timers']
            writer.writerow(line)


def main():
    parser = argparse.ArgumentParser("Create a csv file from a json results file")
    parser.add_argument('result_file', help="The orignial result file")
    parser.add_argument('output_file', help="The output path for the csv file")
    parser.add_argument('-d', '--delimiter', type=str, help="The delimiter for the csv file", default=';')
    args = parser.parse_args()

    results_to_csv(args.result_file, args.output_file, args.delimiter)



import six
import time
import json
import numpy as np
from collections import defaultdict
from oct.results.models import Result, Turret


def split_series(points, interval):
    offset = points[0][0]
    maxval = int((points[-1][0] - offset) // interval)
    vals = defaultdict(list)
    for key, value in points:
        vals[(key - offset) // interval].append(value)
    series = [vals[i] for i in range(maxval + 1)]
    return series


class Results(object):
    def __init__(self, results_file_name, run_time):
        self.results_file_name = results_file_name
        self.run_time = run_time
        self.total_transactions = 0
        self.total_errors = 0
        self.uniq_timer_names = set()
        self.uniq_user_group_names = set()
        self.turrets = []

        self.resp_stats_list = self.__parse_file()

        if len(self.resp_stats_list) > 0:
            self.epoch_start = self.resp_stats_list[0].epoch_secs
            self.epoch_finish = self.resp_stats_list[-1].epoch_secs
            self.start_datetime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.epoch_start))
            self.finish_datetime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.epoch_finish))

    def __parse_file(self):
        resp_stats_list = []
        for item in Result.select().order_by(Result.epoch.asc()):
            if item.error:
                self.total_errors += 1
            self.total_transactions += 1
            if item.custom_timers:
                custom_timers = json.loads(item.custom_timers)
            else:
                custom_timers = {}
            for name, value in six.iteritems(custom_timers):
                self.uniq_timer_names.add(name)
            r = ResponseStats(item.elapsed, item.epoch, item.turret_name, item.scriptrun_time,
                              item.error, custom_timers)
            resp_stats_list.append(r)

        for turret in Turret.select():
            self.turrets.append(turret.to_dict())

        return resp_stats_list


class ResponseStats(object):
    def __init__(self, elapsed_time, epoch_secs, user_group_name, trans_time, error, custom_timers):
        self.elapsed_time = elapsed_time
        self.epoch_secs = epoch_secs
        self.user_group_name = user_group_name
        self.trans_time = trans_time
        self.error = error
        self.custom_timers = custom_timers


class IntervalDetailsResults(object):

    def __init__(self, trans_timer_points, interval_secs):
        self.avg_resptime_points = {}
        self.percentile_80 = {}
        self.percentile_90 = {}
        self.percentile_95 = {}
        self.max = {}
        self.min = {}
        self.avg = {}
        self.stdev = {}
        self.rate = {}
        self.count = {}
        self.interval = interval_secs
        self.interval_list = []
        self.splat_series = split_series(trans_timer_points, interval_secs)
        self.process()

    def process(self):
        for i, bucket in enumerate(self.splat_series):
            interval_start = int((i + 1) * self.interval)
            count = len(bucket)

            if count == 0:
                continue
            else:
                rate = count / float(self.interval)
                min_trans = min(bucket)
                max_trans = max(bucket)
                avg_trans = np.average(bucket)
                pct_80 = np.percentile(bucket, 80)
                pct_90 = np.percentile(bucket, 90)
                pct_95 = np.percentile(bucket, 95)
                stdev = np.std(bucket)

                self.avg_resptime_points[interval_start] = avg_trans
                self.percentile_80[interval_start] = pct_80
                self.percentile_90[interval_start] = pct_90
                self.percentile_95[interval_start] = pct_95
                self.max[interval_start] = max_trans
                self.min[interval_start] = min_trans
                self.stdev[interval_start] = stdev
                self.rate[interval_start] = rate
                self.count[interval_start] = count
                self.interval_list.append(interval_start)


class ReportResults(object):
    def __init__(self, results, interval_secs):
        self.all_results = {}
        self.results = results
        self.custom_timers = []
        self.interval = interval_secs

    def set_dict(self, trans_timer_points, trans_timer_vals, name=None):
        data = {
            'min_trans_val': np.average(trans_timer_vals),
            'average_trans_val': np.average(trans_timer_vals),
            'pct_80_trans_val': np.percentile(trans_timer_vals, 80),
            'pct_90_trans_val': np.percentile(trans_timer_vals, 90),
            'pct_95_trans_val': np.percentile(trans_timer_vals, 95),
            'max_trans_val': max(trans_timer_vals),
            'stdev_trans_val': np.std(trans_timer_vals),
            'interval_results': IntervalDetailsResults(trans_timer_points, self.interval),
            'throughput_points': {},
            'trans_timer_points': trans_timer_points,
            'trans_timer_vals': trans_timer_vals
        }

        for i, bucket in enumerate(data['interval_results'].splat_series):
            data['throughput_points'][int((i + 1) * self.interval)] = len(bucket) / self.interval

        if name is not None:
            data['name'] = name

        return data

    def set_all_transactions_results(self):
        resp_stats_list = self.results.resp_stats_list
        trans_timer_points = []
        trans_timer_vals = []
        for resp_stats in resp_stats_list:
            points = (resp_stats.elapsed_time, resp_stats.trans_time)
            trans_timer_points.append(points)
            trans_timer_vals.append(resp_stats.trans_time)

        self.all_results = self.set_dict(trans_timer_points, trans_timer_vals)

    def clear_all_transactions(self):
        del self.all_results

    def set_custom_timers(self):
        resp_stats_list = self.results.resp_stats_list
        for timer_name in sorted(self.results.uniq_timer_names):
            custom_timer_points = []
            custom_timer_vals = []
            for resp_stats in resp_stats_list:
                try:
                    val = resp_stats.custom_timers[timer_name]
                    custom_timer_points.append((resp_stats.elapsed_time, val))
                    custom_timer_vals.append(val)
                except KeyError:
                    pass  # the timer has never been used

            self.custom_timers.append(self.set_dict(custom_timer_points, custom_timer_vals, timer_name))



class OctConfigurationError(Exception):
    """Provide an oct configuration error
    """
    pass


class OctGenericException(Exception):
    """
    Provide generic exception for reports

    """
    pass


class FormNotFoundException(Exception):
    """
    Raised in case of FormNotFound with browser

    """
    pass


class NoUrlOpen(Exception):
    """
    Raised in case of no url open but requested inside browser class

    """
    pass


class LinkNotFound(Exception):
    """
    Raised in case of link not found in current html document

    """
    pass


class NoFormWaiting(Exception):
    """
    Raised in case of action required form if no form selected

    """
    pass



import os
import pygal


# response time graph for raw data
def resp_graph_raw(nested_resp_list, image_name, dir='./'):
    fig = pygal.XY(stroke=False, x_title='Elapsed Time In Test (secs)',
                   y_title='Response Time (secs)',
                   js=('http://kozea.github.io/pygal.js/2.0.x/pygal-tooltips.min.js',))
    fig.title = image_name.split('.')[0]
    fig.add('Time', [(round(item[0], 2), round(item[1], 2)) for item in nested_resp_list])
    fig.x_labels = map(str, range(1, len([item[1] for item in nested_resp_list]) + 1))
    fig.render_to_file(filename=os.path.join(dir, image_name))


# response time graph for bucketed data
def resp_graph(avg_resptime_points_dict, percentile_80_resptime_points_dict,
               percentile_90_resptime_points_dict, image_name, dir='./'):
    fig = pygal.XY(human_readable=True, x_title='Elapsed Time In Test (secs)',
                   y_title='Response Time (secs)',
                   js=('http://kozea.github.io/pygal.js/2.0.x/pygal-tooltips.min.js',))
    x_seq = sorted(avg_resptime_points_dict.keys())
    fig.add('AVG', [(0, None)] + [(round(x, 2), round(avg_resptime_points_dict[x], 2)) for x in x_seq])
    x_seq = sorted(percentile_90_resptime_points_dict.keys())
    fig.add('90pct', [(0, None)] + [(round(x, 2), round(percentile_90_resptime_points_dict[x], 2)) for x in x_seq])
    x_seq = sorted(percentile_80_resptime_points_dict.keys())
    fig.add('80pct', [(0, None)] + [(round(x, 2), round(percentile_80_resptime_points_dict[x], 2)) for x in x_seq])
    fig.render_to_file(filename=os.path.join(dir, image_name))


# throughput graph
def tp_graph(throughputs_dict, image_name, dir='./'):
    fig = pygal.XY(x_title='Elapsed Time In Test (secs)', y_title='Transactions Per Second (count)',
                   human_readable=True, js=('http://kozea.github.io/pygal.js/2.0.x/pygal-tooltips.min.js',))
    x_seq = sorted(throughputs_dict.keys())
    y_seq = [round(throughputs_dict[x], 2) for x in x_seq]
    fig.add('Transactions per second', [(None, 0)] + list(zip(x_seq, y_seq)))
    fig.render_to_file(filename=os.path.join(dir, image_name))



import argparse

from oct.results.resultsoutput import output
from oct.results.models import db, set_database
from oct.utilities.configuration import configure


def main():
    parser = argparse.ArgumentParser("Rebuild the html report from result dir")
    parser.add_argument('results_dir', help="The directory containing the results")
    parser.add_argument('results_file', help="The result file")
    parser.add_argument('config_file', help="The configuration file")

    args = parser.parse_args()

    config = configure(None, None, args.config_file)
    set_database(args.results_file, db, config)
    output(args.results_dir, args.results_file, config)



import json
import datetime
from peewee import Proxy, TextField, FloatField, CharField, IntegerField, SqliteDatabase, Model, DateTimeField

db = Proxy()


class Result(Model):
    error = TextField(null=True)
    scriptrun_time = FloatField()
    elapsed = FloatField()
    epoch = FloatField()
    custom_timers = TextField(null=True)
    turret_name = CharField(default='Noname')

    def to_dict(self):
        return {
            'error': self.error,
            'scriptrun_time': self.scriptrun_time,
            'elapsed': self.elapsed,
            'epoch': self.epoch,
            'custom_timers': json.loads(self.custom_timers),
            'turret_name': self.turret_name
        }

    class Meta:
        database = db


class Turret(Model):
    name = TextField()
    uuid = TextField()
    canons = IntegerField()
    script = TextField()
    rampup = IntegerField()
    status = TextField()
    updated_at = DateTimeField(default=datetime.datetime.now())

    def save(self, *args, **kwargs):
        self.updated_at = datetime.datetime.now()
        return super(Turret, self).save(*args, **kwargs)

    def to_dict(self):
        return {
            'name': self.name,
            'uuid': self.uuid,
            'canons': self.canons,
            'script': self.script,
            'rampup': self.rampup,
            'status': self.status,
            'updated_at': self.updated_at
        }

    class Meta:
        database = db


def set_database(db_path, proxy, config):
    """Initialize the peewee database with the given configuration

    :param db_path str: the path of the sqlite database
    :param proxy peewee.Proxy: the peewee proxy to initialise
    :param config dict: the configuration dictionnary
    """
    if 'testing' in config and config['testing'] is True:
        database = SqliteDatabase('/tmp/results.sqlite', check_same_thread=False)
    else:
        database = SqliteDatabase(db_path, check_same_thread=False)
    proxy.initialize(database)



from __future__ import print_function
import os
import shutil
import sys
from jinja2 import Environment, PackageLoader

BASE_DIR = os.path.abspath(os.path.dirname(__file__))


def create_project(project_name):

    env = Environment(loader=PackageLoader('oct.utilities', 'templates'))

    config_content = env.get_template('configuration/config.json').render(script_name='v_user.py')
    script_content = env.get_template('scripts/v_user.py').render()

    if os.path.exists(project_name):
        sys.stderr.write('\nERROR: project already exists: %s\n\n' % project_name)
        raise OSError("Project %s already exists" % project_name)
    try:
        os.makedirs(project_name)
        os.makedirs(os.path.join(project_name, 'test_scripts'))
        os.makedirs(os.path.join(project_name, 'templates'))
        os.makedirs(os.path.join(project_name, 'templates', 'css'))
        os.makedirs(os.path.join(project_name, 'templates', 'scripts'))
        os.makedirs(os.path.join(project_name, 'templates', 'img'))
        shutil.copy(os.path.join(BASE_DIR, 'templates', 'css', 'style.css'),
                    os.path.join(project_name, 'templates', 'css'))
        shutil.copy(os.path.join(BASE_DIR, 'templates', 'html', 'report.html'),
                    os.path.join(project_name, 'templates'))
    except OSError:
        print('ERROR: can not create directory for %r' % project_name, file=sys.stderr)
        raise
    with open(os.path.join(project_name, 'config.json'), 'w') as f:
        f.write(config_content)
    with open(os.path.join(project_name, 'test_scripts', 'v_user.py'), 'w') as f:
        f.write(script_content)


def main():
    try:
        project_name = sys.argv[1]
    except IndexError:
        print('ERROR: no project specified', file=sys.stderr)
        print('Usage: oct-newproject <project name>')
        raise

    create_project(project_name)
